@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) The Exim Maintainers 2023 */
+/* Copyright (c) The Exim Maintainers 2023 - 2024 */
 /* Copyright (c) Jeremy Harris 2019-2020 */
 /* See the file NOTICE for conditions of use and distribution. */
 /* SPDX-License-Identifier: GPL-2.0-or-later */
@@ -14,6 +14,8 @@ method defined in RFC 4422 Appendix A.
 
 
 #include "../exim.h"
+
+#ifdef AUTH_EXTERNAL	/* Remainder of file */
 #include "external.h"
 
 /* Options specific to the external authentication mechanism. */
@@ -153,5 +155,6 @@ return OK;
 
 
 
-#endif   /*!MACRO_PREDEF*/
+#endif	/*!MACRO_PREDEF*/
+#endif	/*AUTH_EXTERNAL*/
 /* End of external.c */
