@@ -2,8 +2,9 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
+/* Copyright (c) The Exim Maintainers 1999 - 2022 */
 /* Copyright (c) 1998 Malcolm Beattie */
-/* Copyright (C) 1999 - 2018  Exim maintainers */
+/* SPDX-License-Identifier: GPL-2.0-or-later */
 
 /* Modified by PH to get rid of the "na" usage, March 1999.
    Modified further by PH for general tidying for Exim 4.
@@ -15,6 +16,10 @@
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include <assert.h>
+
+#define HINTSDB_H
+#define DBFUNCTIONS_H
+
 #include "exim.h"
 
 #define EXIM_TRUE TRUE
